///<reference path="../../../.d.ts"/>
"use strict";
var Future = require("fibers/future");
var IosEmulatorServices = (function () {
    function IosEmulatorServices($logger, $emulatorSettingsService, $errors, $childProcess, $mobileHelper, $devicePlatformsConstants, $hostInfo, $options) {
        this.$logger = $logger;
        this.$emulatorSettingsService = $emulatorSettingsService;
        this.$errors = $errors;
        this.$childProcess = $childProcess;
        this.$mobileHelper = $mobileHelper;
        this.$devicePlatformsConstants = $devicePlatformsConstants;
        this.$hostInfo = $hostInfo;
        this.$options = $options;
    }
    IosEmulatorServices.prototype.checkDependencies = function () {
        return (function () {
        }).future()();
    };
    IosEmulatorServices.prototype.checkAvailability = function (dependsOnProject) {
        var _this = this;
        if (dependsOnProject === void 0) { dependsOnProject = true; }
        return (function () {
            if (!_this.$hostInfo.isDarwin) {
                _this.$errors.fail("iOS Simulator is available only on Mac OS X.");
            }
            var platform = _this.$devicePlatformsConstants.iOS;
            if (dependsOnProject && !_this.$emulatorSettingsService.canStart(platform).wait()) {
                _this.$errors.fail("The current project does not target iOS and cannot be run in the iOS Simulator.");
            }
        }).future()();
    };
    IosEmulatorServices.prototype.startEmulator = function (app, emulatorOptions) {
        var _this = this;
        return (function () {
            _this.killLaunchdSim().wait();
            _this.startEmulatorCore(app, emulatorOptions);
        }).future()();
    };
    IosEmulatorServices.prototype.postDarwinNotification = function (notification) {
        var iosSimPath = require.resolve("ios-sim-portable");
        var nodeCommandName = process.argv[0];
        var opts = ["notify-post", notification];
        if (this.$options.device) {
            opts.push("--device", this.$options.device);
        }
        return this.$childProcess.exec(nodeCommandName + " " + iosSimPath + " " + opts.join(' '));
    };
    IosEmulatorServices.prototype.killLaunchdSim = function () {
        this.$logger.info("Cleaning up before starting the iOS Simulator");
        var future = new Future();
        var killAllProc = this.$childProcess.spawn("killall", ["launchd_sim"]);
        killAllProc.on("close", function (code) {
            future.return();
        });
        return future;
    };
    IosEmulatorServices.prototype.startEmulatorCore = function (app, emulatorOptions) {
        this.$logger.info("Starting iOS Simulator");
        var iosSimPath = require.resolve("ios-sim-portable");
        var nodeCommandName = process.argv[0];
        if (this.$options.availableDevices) {
            this.$childProcess.spawnFromEvent(nodeCommandName, [iosSimPath, "device-types"], "close", { stdio: "inherit" }).wait();
            return;
        }
        var opts = [
            iosSimPath,
            "launch", app,
            "--timeout", this.$options.timeout
        ];
        if (!this.$options.justlaunch) {
            opts.push("--logging");
        }
        else {
            if (emulatorOptions) {
                if (emulatorOptions.stderrFilePath) {
                    opts = opts.concat("--stderr", emulatorOptions.stderrFilePath);
                }
                if (emulatorOptions.stdoutFilePath) {
                    opts = opts.concat("--stdout", emulatorOptions.stdoutFilePath);
                }
            }
            opts.push("--exit");
        }
        if (this.$options.device) {
            opts = opts.concat("--device", this.$options.device);
        }
        if (emulatorOptions && emulatorOptions.args) {
            opts.push("--args=" + emulatorOptions.args);
        }
        this.$childProcess.spawn(nodeCommandName, opts, { stdio: "inherit" });
    };
    return IosEmulatorServices;
})();
$injector.register("iOSEmulatorServices", IosEmulatorServices);
