///<reference path="../.d.ts"/>
"use strict";
var __extends = this.__extends || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    __.prototype = b.prototype;
    d.prototype = new __();
};
var androidLiveSyncServiceLib = require("../common/mobile/android/android-livesync-service");
var constants = require("../constants");
var usbLivesyncServiceBaseLib = require("../common/services/usb-livesync-service-base");
var path = require("path");
var UsbLiveSyncService = (function (_super) {
    __extends(UsbLiveSyncService, _super);
    function UsbLiveSyncService($commandsService, $devicesServices, $fs, $mobileHelper, $localToDevicePathDataFactory, $options, $platformsData, $projectData, $deviceAppDataFactory, $logger, $injector, $platformService, $dispatcher) {
        _super.call(this, $devicesServices, $mobileHelper, $localToDevicePathDataFactory, $logger, $options, $deviceAppDataFactory, $fs, $dispatcher);
        this.$commandsService = $commandsService;
        this.$platformsData = $platformsData;
        this.$projectData = $projectData;
        this.$injector = $injector;
        this.$platformService = $platformService;
        this.excludedProjectDirsAndFiles = [
            "app_resources"
        ];
    }
    UsbLiveSyncService.prototype.liveSync = function (platform) {
        var _this = this;
        return (function () {
            platform = platform || _this.initialize(platform).wait();
            _this.$platformService.preparePlatform(platform).wait();
            var platformData = _this.$platformsData.getPlatformData(platform.toLowerCase());
            var projectFilesPath = path.join(platformData.appDestinationDirectoryPath, constants.APP_FOLDER_NAME);
            var canLiveSyncAction = function (device, appIdentifier) {
                return (function () {
                    if (platform.toLowerCase() === "android") {
                        var output = device.adb.executeShellCommand("\"echo '' | run-as " + appIdentifier + "\"").wait();
                        if (output.indexOf("run-as: Package '" + appIdentifier + "' is unknown") !== -1) {
                            _this.$logger.warn("Unable to livesync on device " + device.deviceInfo.identifier + ". Consider upgrading your device OS.");
                            return false;
                        }
                    }
                    return true;
                }).future()();
            };
            var restartAppOnDeviceAction = function (device, deviceAppData, localToDevicePaths) {
                var platformSpecificUsbLiveSyncService = _this.resolveUsbLiveSyncService(platform || _this.$devicesServices.platform, device);
                return platformSpecificUsbLiveSyncService.restartApplication(deviceAppData, localToDevicePaths);
            };
            var notInstalledAppOnDeviceAction = function (device) {
                return _this.$platformService.deployOnDevice(platform);
            };
            var beforeBatchLiveSyncAction = function (filePath) {
                return (function () {
                    _this.$platformService.preparePlatform(platform).wait();
                    return path.join(projectFilesPath, path.relative(path.join(_this.$projectData.projectDir, constants.APP_FOLDER_NAME), filePath));
                }).future()();
            };
            var watchGlob = path.join(_this.$projectData.projectDir, constants.APP_FOLDER_NAME) + "/**/*";
            _this.sync(platform, _this.$projectData.projectId, platformData.appDestinationDirectoryPath, projectFilesPath, _this.excludedProjectDirsAndFiles, watchGlob, restartAppOnDeviceAction, notInstalledAppOnDeviceAction, beforeBatchLiveSyncAction, canLiveSyncAction).wait();
        }).future()();
    };
    UsbLiveSyncService.prototype.resolveUsbLiveSyncService = function (platform, device) {
        var platformSpecificUsbLiveSyncService = null;
        if (platform.toLowerCase() === "android") {
            platformSpecificUsbLiveSyncService = this.$injector.resolve(AndroidUsbLiveSyncService, { _device: device });
        }
        else if (platform.toLowerCase() === "ios") {
            platformSpecificUsbLiveSyncService = this.$injector.resolve(IOSUsbLiveSyncService, { _device: device });
        }
        return platformSpecificUsbLiveSyncService;
    };
    return UsbLiveSyncService;
})(usbLivesyncServiceBaseLib.UsbLiveSyncServiceBase);
exports.UsbLiveSyncService = UsbLiveSyncService;
$injector.register("usbLiveSyncService", UsbLiveSyncService);
var IOSUsbLiveSyncService = (function () {
    function IOSUsbLiveSyncService(_device) {
        this._device = _device;
    }
    Object.defineProperty(IOSUsbLiveSyncService.prototype, "device", {
        get: function () {
            return this._device;
        },
        enumerable: true,
        configurable: true
    });
    IOSUsbLiveSyncService.prototype.restartApplication = function (deviceAppData) {
        return this.device.applicationManager.restartApplication(deviceAppData.appIdentifier);
    };
    return IOSUsbLiveSyncService;
})();
exports.IOSUsbLiveSyncService = IOSUsbLiveSyncService;
var AndroidUsbLiveSyncService = (function (_super) {
    __extends(AndroidUsbLiveSyncService, _super);
    function AndroidUsbLiveSyncService(_device, $fs, $mobileHelper, $options) {
        _super.call(this, _device, $fs, $mobileHelper);
        this.$options = $options;
    }
    AndroidUsbLiveSyncService.prototype.restartApplication = function (deviceAppData, localToDevicePaths) {
        var _this = this;
        return (function () {
            if (_this.$options.companion) {
                var commands = [_this.liveSyncCommands.SyncFilesCommand()];
                _this.livesync(deviceAppData.appIdentifier, deviceAppData.deviceProjectRootPath, commands).wait();
            }
            else {
                _this.device.adb.executeShellCommand("chmod 0777 " + _this.$mobileHelper.buildDevicePath(deviceAppData.deviceProjectRootPath, "app")).wait();
                var commands = [];
                var devicePathRoot = "/data/data/" + deviceAppData.appIdentifier + "/files";
                _.each(localToDevicePaths, function (localToDevicePath) {
                    var devicePath = _this.$mobileHelper.correctDevicePath(path.join(devicePathRoot, localToDevicePath.getRelativeToProjectBasePath()));
                    if (_this.$fs.getFsStats(localToDevicePath.getLocalPath()).wait().isFile()) {
                        commands.push("mv \"" + localToDevicePath.getDevicePath() + "\" \"" + devicePath + "\"");
                    }
                });
                commands.push("rm -rf " + _this.$mobileHelper.buildDevicePath(devicePathRoot, "code_cache", "secondary_dexes", "proxyThumb"));
                commands.push("exit");
                var commandsFileDevicePath = _this.$mobileHelper.buildDevicePath(deviceAppData.deviceProjectRootPath, AndroidUsbLiveSyncService.LIVESYNC_COMMANDS_FILE_NAME);
                _this.createCommandsFileOnDevice(commandsFileDevicePath, commands).wait();
                var result = _this.device.adb.executeShellCommand("\"cat " + commandsFileDevicePath + " | run-as " + deviceAppData.appIdentifier + "\"").wait();
                if (result.indexOf("Permission denied") !== -1) {
                    _this.device.adb.executeShellCommand("" + commandsFileDevicePath).wait();
                }
            }
            _this.device.applicationManager.restartApplication(deviceAppData.appIdentifier).wait();
        }).future()();
    };
    AndroidUsbLiveSyncService.LIVESYNC_COMMANDS_FILE_NAME = "nativescript.livesync.commands.sh";
    return AndroidUsbLiveSyncService;
})(androidLiveSyncServiceLib.AndroidLiveSyncService);
exports.AndroidUsbLiveSyncService = AndroidUsbLiveSyncService;
